@@ -1,6 +1,7 @@
+//! Functionality to compress and decompress data according to the parquet specification
 pub use super::parquet_bridge::Compression;
 
-use crate::error::{ParquetError, Result};
+use crate::error::{Error, Result};
 
 /// Compresses data stored in slice `input_buf` and writes the compressed result
 /// to `output_buf`.
@@ -29,7 +30,7 @@ pub fn compress(
             encoder.flush().map_err(|e| e.into())
         }
         #[cfg(not(feature = "brotli"))]
-        Compression::Brotli => Err(ParquetError::FeatureNotActive(
+        Compression::Brotli => Err(Error::FeatureNotActive(
             crate::error::Feature::Brotli,
             "compress to brotli".to_string(),
         )),
@@ -41,7 +42,7 @@ pub fn compress(
             encoder.try_finish().map_err(|e| e.into())
         }
         #[cfg(not(feature = "gzip"))]
-        Compression::Gzip => Err(ParquetError::FeatureNotActive(
+        Compression::Gzip => Err(Error::FeatureNotActive(
             crate::error::Feature::Gzip,
             "compress to gzip".to_string(),
         )),
@@ -57,28 +58,37 @@ pub fn compress(
             Ok(())
         }
         #[cfg(not(feature = "snappy"))]
-        Compression::Snappy => Err(ParquetError::FeatureNotActive(
+        Compression::Snappy => Err(Error::FeatureNotActive(
             crate::error::Feature::Snappy,
             "compress to snappy".to_string(),
         )),
-        #[cfg(feature = "lz4")]
-        Compression::Lz4 => {
-            use std::io::Write;
-            const LZ4_BUFFER_SIZE: usize = 4096;
-            let mut encoder = lz4::EncoderBuilder::new().build(output_buf)?;
-            let mut from = 0;
-            loop {
-                let to = std::cmp::min(from + LZ4_BUFFER_SIZE, input_buf.len());
-                encoder.write_all(&input_buf[from..to])?;
-                from += LZ4_BUFFER_SIZE;
-                if from >= input_buf.len() {
-                    break;
-                }
-            }
-            encoder.finish().1.map_err(|e| e.into())
+        #[cfg(all(feature = "lz4_flex", not(feature = "lz4")))]
+        Compression::Lz4Raw => {
+            let output_buf_len = output_buf.len();
+            let required_len = lz4_flex::block::get_maximum_output_size(input_buf.len());
+            output_buf.resize(output_buf_len + required_len, 0);
+
+            let compressed_size =
+                lz4_flex::block::compress_into(input_buf, &mut output_buf[output_buf_len..])?;
+            output_buf.truncate(output_buf_len + compressed_size);
+            Ok(())
         }
-        #[cfg(not(feature = "lz4"))]
-        Compression::Lz4 => Err(ParquetError::FeatureNotActive(
+        #[cfg(feature = "lz4")]
+        Compression::Lz4Raw => {
+            let output_buf_len = output_buf.len();
+            let required_len = lz4::block::compress_bound(input_buf.len())?;
+            output_buf.resize(required_len, 0);
+            let size = lz4::block::compress_to_buffer(
+                input_buf,
+                None,
+                false,
+                &mut output_buf[output_buf_len..],
+            )?;
+            output_buf.truncate(output_buf_len + size);
+            Ok(())
+        }
+        #[cfg(all(not(feature = "lz4"), not(feature = "lz4_flex")))]
+        Compression::Lz4Raw => Err(Error::FeatureNotActive(
             crate::error::Feature::Lz4,
             "compress to lz4".to_string(),
         )),
@@ -96,7 +106,7 @@ pub fn compress(
             }
         }
         #[cfg(not(feature = "zstd"))]
-        Compression::Zstd => Err(ParquetError::FeatureNotActive(
+        Compression::Zstd => Err(Error::FeatureNotActive(
             crate::error::Feature::Zstd,
             "compress to zstd".to_string(),
         )),
@@ -123,7 +133,7 @@ pub fn decompress(compression: Compression, input_buf: &[u8], output_buf: &mut [
                 .map_err(|e| e.into())
         }
         #[cfg(not(feature = "brotli"))]
-        Compression::Brotli => Err(ParquetError::FeatureNotActive(
+        Compression::Brotli => Err(Error::FeatureNotActive(
             crate::error::Feature::Brotli,
             "decompress with brotli".to_string(),
         )),
@@ -134,7 +144,7 @@ pub fn decompress(compression: Compression, input_buf: &[u8], output_buf: &mut [
             decoder.read_exact(output_buf).map_err(|e| e.into())
         }
         #[cfg(not(feature = "gzip"))]
-        Compression::Gzip => Err(ParquetError::FeatureNotActive(
+        Compression::Gzip => Err(Error::FeatureNotActive(
             crate::error::Feature::Gzip,
             "decompress with gzip".to_string(),
         )),
@@ -150,18 +160,22 @@ pub fn decompress(compression: Compression, input_buf: &[u8], output_buf: &mut [
                 .map(|_| ())
         }
         #[cfg(not(feature = "snappy"))]
-        Compression::Snappy => Err(ParquetError::FeatureNotActive(
+        Compression::Snappy => Err(Error::FeatureNotActive(
             crate::error::Feature::Snappy,
             "decompress with snappy".to_string(),
         )),
+        #[cfg(all(feature = "lz4_flex", not(feature = "lz4")))]
+        Compression::Lz4Raw => lz4_flex::block::decompress_into(input_buf, output_buf)
+            .map(|_| {})
+            .map_err(|e| e.into()),
         #[cfg(feature = "lz4")]
-        Compression::Lz4 => {
-            use std::io::Read;
-            let mut decoder = lz4::Decoder::new(input_buf)?;
-            decoder.read_exact(output_buf).map_err(|e| e.into())
+        Compression::Lz4Raw => {
+            lz4::block::decompress_to_buffer(input_buf, Some(output_buf.len() as i32), output_buf)
+                .map(|_| {})
+                .map_err(|e| e.into())
         }
-        #[cfg(not(feature = "lz4"))]
-        Compression::Lz4 => Err(ParquetError::FeatureNotActive(
+        #[cfg(all(not(feature = "lz4"), not(feature = "lz4_flex")))]
+        Compression::Lz4Raw => Err(Error::FeatureNotActive(
             crate::error::Feature::Lz4,
             "decompress with lz4".to_string(),
         )),
@@ -172,7 +186,7 @@ pub fn decompress(compression: Compression, input_buf: &[u8], output_buf: &mut [
             decoder.read_exact(output_buf).map_err(|e| e.into())
         }
         #[cfg(not(feature = "zstd"))]
-        Compression::Zstd => Err(ParquetError::FeatureNotActive(
+        Compression::Zstd => Err(Error::FeatureNotActive(
             crate::error::Feature::Zstd,
             "decompress with zstd".to_string(),
         )),
@@ -230,7 +244,7 @@ mod tests {
 
     #[test]
     fn test_codec_lz4() {
-        test_codec(Compression::Lz4);
+        test_codec(Compression::Lz4Raw);
     }
 
     #[test]
