@@ -3,20 +3,17 @@ use std::sync::Arc;
 use parquet_format_async_temp::Statistics as ParquetStatistics;
 
 use super::Statistics;
-use crate::metadata::ColumnDescriptor;
+use crate::error::{Error, Result};
+use crate::schema::types::{PhysicalType, PrimitiveType};
 use crate::types;
-use crate::{
-    error::{ParquetError, Result},
-    schema::types::PhysicalType,
-};
 
 #[derive(Debug, Clone, PartialEq)]
 pub struct PrimitiveStatistics<T: types::NativeType> {
-    pub descriptor: ColumnDescriptor,
+    pub primitive_type: PrimitiveType,
     pub null_count: Option<i64>,
     pub distinct_count: Option<i64>,
-    pub max_value: Option<T>,
     pub min_value: Option<T>,
+    pub max_value: Option<T>,
 }
 
 impl<T: types::NativeType> Statistics for PrimitiveStatistics<T> {
@@ -35,25 +32,25 @@ impl<T: types::NativeType> Statistics for PrimitiveStatistics<T> {
 
 pub fn read<T: types::NativeType>(
     v: &ParquetStatistics,
-    descriptor: ColumnDescriptor,
+    primitive_type: PrimitiveType,
 ) -> Result<Arc<dyn Statistics>> {
     if let Some(ref v) = v.max_value {
         if v.len() != std::mem::size_of::<T>() {
-            return Err(ParquetError::OutOfSpec(
+            return Err(Error::OutOfSpec(
                 "The max_value of statistics MUST be plain encoded".to_string(),
             ));
         }
     };
     if let Some(ref v) = v.min_value {
         if v.len() != std::mem::size_of::<T>() {
-            return Err(ParquetError::OutOfSpec(
+            return Err(Error::OutOfSpec(
                 "The min_value of statistics MUST be plain encoded".to_string(),
             ));
         }
     };
 
     Ok(Arc::new(PrimitiveStatistics::<T> {
-        descriptor,
+        primitive_type,
         null_count: v.null_count,
         distinct_count: v.distinct_count,
         max_value: v.max_value.as_ref().map(|x| types::decode(x)),
