@@ -4,14 +4,14 @@ use futures::AsyncWrite;
 use parquet_format_async_temp::{ColumnChunk, RowGroup};
 
 use crate::{
-    compression::Compression,
-    error::{ParquetError, Result},
+    error::{Error, Result},
     metadata::{ColumnChunkMetaData, ColumnDescriptor},
     page::CompressedPage,
 };
 
 use super::{
     column_chunk::{write_column_chunk, write_column_chunk_async},
+    page::{is_data_page, PageWriteSpec},
     DynIter, DynStreamingIterator,
 };
 
@@ -51,6 +51,27 @@ impl ColumnOffsetsMetadata {
     }
 }
 
+fn compute_num_rows(columns: &[(ColumnChunk, Vec<PageWriteSpec>)]) -> Result<i64> {
+    columns
+        .get(0)
+        .map(|(_, specs)| {
+            let mut num_rows = 0;
+            specs
+                .iter()
+                .filter(|x| is_data_page(x))
+                .try_for_each(|spec| {
+                    num_rows += spec.num_rows.ok_or_else(|| {
+                        Error::OutOfSpec(
+                            "All data pages must declare the number of rows on it".to_string(),
+                        )
+                    })? as i64;
+                    Result::Ok(())
+                })?;
+            Result::Ok(num_rows)
+        })
+        .unwrap_or(Ok(0))
+}
+
 pub fn write_row_group<
     'a,
     W,
@@ -59,13 +80,12 @@ pub fn write_row_group<
     writer: &mut W,
     mut offset: u64,
     descriptors: &[ColumnDescriptor],
-    compression: Compression,
     columns: DynIter<'a, std::result::Result<DynStreamingIterator<'a, CompressedPage, E>, E>>,
-    num_rows: usize,
-) -> Result<(RowGroup, u64)>
+    ordinal: usize,
+) -> Result<(RowGroup, Vec<Vec<PageWriteSpec>>, u64)>
 where
     W: Write,
-    ParquetError: From<E>,
+    Error: From<E>,
     E: std::error::Error,
 {
     let column_iter = descriptors.iter().zip(columns);
@@ -73,38 +93,46 @@ where
     let initial = offset;
     let columns = column_iter
         .map(|(descriptor, page_iter)| {
-            let (column, size) =
-                write_column_chunk(writer, offset, descriptor, compression, page_iter?)?;
+            let (column, page_specs, size) =
+                write_column_chunk(writer, offset, descriptor, page_iter?)?;
             offset += size;
-            Ok(column)
+            Ok((column, page_specs))
         })
         .collect::<Result<Vec<_>>>()?;
     let bytes_written = offset - initial;
 
+    let num_rows = compute_num_rows(&columns)?;
+
     // compute row group stats
     let file_offset = columns
-        .iter()
-        .next()
-        .map(|column_chunk| {
+        .get(0)
+        .map(|(column_chunk, _)| {
             ColumnOffsetsMetadata::from_column_chunk(column_chunk).calc_row_group_file_offset()
         })
         .unwrap_or(None);
 
     let total_byte_size = columns
         .iter()
-        .map(|c| c.meta_data.as_ref().unwrap().total_compressed_size)
+        .map(|(c, _)| c.meta_data.as_ref().unwrap().total_uncompressed_size)
         .sum();
+    let total_compressed_size = columns
+        .iter()
+        .map(|(c, _)| c.meta_data.as_ref().unwrap().total_compressed_size)
+        .sum();
+
+    let (columns, specs) = columns.into_iter().unzip();
 
     Ok((
         RowGroup {
             columns,
             total_byte_size,
-            num_rows: num_rows as i64,
+            num_rows,
             sorting_columns: None,
             file_offset,
-            total_compressed_size: None,
-            ordinal: None,
+            total_compressed_size: Some(total_compressed_size),
+            ordinal: ordinal.try_into().ok(),
         },
+        specs,
         bytes_written,
     ))
 }
@@ -117,13 +145,11 @@ pub async fn write_row_group_async<
     writer: &mut W,
     mut offset: u64,
     descriptors: &[ColumnDescriptor],
-    compression: Compression,
     columns: DynIter<'a, std::result::Result<DynStreamingIterator<'a, CompressedPage, E>, E>>,
-    num_rows: usize,
-) -> Result<(RowGroup, u64)>
+) -> Result<(RowGroup, Vec<Vec<PageWriteSpec>>, u64)>
 where
     W: AsyncWrite + Unpin + Send,
-    ParquetError: From<E>,
+    Error: From<E>,
     E: std::error::Error,
 {
     let column_iter = descriptors.iter().zip(columns);
@@ -131,26 +157,33 @@ where
     let initial = offset;
     let mut columns = vec![];
     for (descriptor, page_iter) in column_iter {
-        let (spec, size) =
-            write_column_chunk_async(writer, offset, descriptor, compression, page_iter?).await?;
-        offset += size as u64;
-        columns.push(spec);
+        let (column, page_specs, size) =
+            write_column_chunk_async(writer, offset, descriptor, page_iter?).await?;
+        offset += size;
+        columns.push((column, page_specs));
     }
     let bytes_written = offset - initial;
 
+    let num_rows = compute_num_rows(&columns)?;
+
     // compute row group stats
-    let file_offest = columns
-        .iter()
-        .next()
-        .map(|column_chunk| {
+    let file_offset = columns
+        .get(0)
+        .map(|(column_chunk, _)| {
             ColumnOffsetsMetadata::from_column_chunk(column_chunk).calc_row_group_file_offset()
         })
         .unwrap_or(None);
 
     let total_byte_size = columns
         .iter()
-        .map(|c| c.meta_data.as_ref().unwrap().total_compressed_size)
+        .map(|(c, _)| c.meta_data.as_ref().unwrap().total_uncompressed_size)
         .sum();
+    let total_compressed_size = columns
+        .iter()
+        .map(|(c, _)| c.meta_data.as_ref().unwrap().total_compressed_size)
+        .sum();
+
+    let (columns, specs) = columns.into_iter().unzip();
 
     Ok((
         RowGroup {
@@ -158,10 +191,11 @@ where
             total_byte_size,
             num_rows: num_rows as i64,
             sorting_columns: None,
-            file_offset: file_offest,
-            total_compressed_size: None,
+            file_offset,
+            total_compressed_size: Some(total_compressed_size),
             ordinal: None,
         },
+        specs,
         bytes_written,
     ))
 }
